"use strict";
/**
 * @module Features
 */
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var error_reason_1 = require("./../../errors/error-reason");
var jmx_fetch_request_1 = require("./../services/fetch-jmx-values/jmx-fetch-request");
var Services = require("./../services/metrics-services");
var measured_entity_class_request_1 = require("./../services/metrics/measured-entity-class-request");
/**
 * Internal metrics feature.
 *
 * <p>Also provides access to JMX MBean values.
 */
var Metrics = /** @class */ (function () {
    function Metrics(internal) {
        this.internal = internal;
    }
    /**
     * Resolve and invoke a service.
     */
    Metrics.prototype.callService = function (serviceDefinition, request) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.internal.checkConnected(reject)) {
                var service = _this.internal.getServiceLocator().obtain(serviceDefinition);
                service.send(request, function (err, response) {
                    if (err) {
                        reject(err);
                    }
                    else {
                        // tslint:disable-next-line:strict-type-predicates
                        resolve((response === null ? undefined : response));
                    }
                });
            }
        });
    };
    /**
     * Add a new session metric collector, removing any with the same name.
     */
    Metrics.prototype.putSessionMetricCollector = function (sessionMetricCollector) {
        return this.callService(Services.PUT_SESSION_METRIC_COLLECTOR, sessionMetricCollector);
    };
    /**
     * Remove any session metric collector with the given name.
     */
    Metrics.prototype.removeSessionMetricCollector = function (sessionMetricCollectorName) {
        return this.callService(Services.REMOVE_SESSION_METRIC_COLLECTOR, sessionMetricCollectorName);
    };
    /**
     * Retrieve all session metric collectors.
     */
    Metrics.prototype.fetchSessionMetricCollectors = function () {
        return this.callService(Services.FETCH_SESSION_METRIC_COLLECTORS, null);
    };
    /**
     * Add a new topic metric collector, removing any with the same name.
     */
    Metrics.prototype.putTopicMetricCollector = function (topicMetricCollector) {
        return this.callService(Services.PUT_TOPIC_METRIC_COLLECTOR, topicMetricCollector);
    };
    /**
     * Remove any topic metric collector with the given name.
     */
    Metrics.prototype.removeTopicMetricCollector = function (topicMetricCollectorName) {
        return this.callService(Services.REMOVE_TOPIC_METRIC_COLLECTOR, topicMetricCollectorName);
    };
    /**
     * Retrieve all topic metric collectors.
     */
    Metrics.prototype.fetchTopicMetricCollectors = function () {
        return this.callService(Services.FETCH_TOPIC_METRIC_COLLECTORS, null);
    };
    /**
     * Get the server-scope metrics.
     */
    Metrics.prototype.getServerMetrics = function () {
        return this.callService(Services.GET_SERVER_METRICS, null);
    };
    /**
     * Get all metrics for a measured entity class. Optionally, a server can be specified.
     *
     * @param measuredEntityClassName e.g. "sessions", "topics"
     */
    Metrics.prototype.getMeasuredEntityClassMetrics = function (entityClass, serverName) {
        return this.callService(Services.GET_MEASURED_ENTITY_CLASS_METRICS, new measured_entity_class_request_1.MeasuredEntityClassRequest(entityClass, serverName));
    };
    /**
     * Get connected server MBean values.
     *
     * @param JMX object names
     */
    Metrics.prototype.fetchJMXValues = function (objectNames) {
        var resultMap = this.callService(Services.FETCH_JMX_VALUES, new jmx_fetch_request_1.JMXFetchRequest(objectNames, false));
        return resultMap.then(function (m) {
            var _a = __read(Object.keys(m)), first = _a[0], more = _a.slice(1);
            var firstResult = m[first];
            if (!more.length && !('reason' in firstResult)) {
                return firstResult;
            }
            throw error_reason_1.ErrorReason.INVALID_DATA;
        });
    };
    /**
     * Get map of server name(s) -> MBean values.
     *
     * @param JMX object names
     */
    Metrics.prototype.fetchClusterJMXValues = function (objectNames) {
        return this.callService(Services.FETCH_JMX_VALUES, new jmx_fetch_request_1.JMXFetchRequest(objectNames, true));
    };
    return Metrics;
}());
exports.Metrics = Metrics;
